/*
 *	LCD interface example
 *	Uses routines from delay.c
 *	This code will interface to a standard LCD controller
 *	like the Hitachi HD44780. It uses it in 4 bit mode, with
 *	the hardware connected as follows (the standard 14 pin 
 *	LCD connector is used):
 *	
 *	PORTB bits 0-3 are connected to the LCD data bits 4-7 (high nibble)
 *	PORTA bit 3 is connected to the LCD RS input (register select)
 *	PORTA bit 5 is connected to the LCD EN bit (enable)
 *	
 *	To use these routines, set up the port I/O (TRISA, TRISB) then
 *	call lcd_init(), then other routines as required.
 *	
 */

#include	<pic18.h>
#include	"lcd.h"
#include	"delay.h"

const unsigned char russian[]={ 	0x41, 0xA0, 0x42, 0xA1, 0xE0, 0x45, 0xA3, 0xA4, 0xA5,
                      					0xA6, 0x4B, 0xA7, 0x4D, 0x48, 0x4F, 0xA8, 0x50, 0x43,
                      					0x54, 0xA9, 0xAA, 0x58, 0xE1, 0xAB, 0xAC, 0xE2, 0xAD,
                     					0xAE, 0x62, 0xAF, 0xB0, 0xB1,
                      					0x61, 0xB2, 0xB3, 0xB4, 0xE3, 0x65, 0xB6, 0xB7, 0xB8,
                      					0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0x6F, 0xBE, 0x70, 0x63,
                      					0xBF, 0x79, 0xE4, 0x78, 0xE5, 0xC0, 0xC1, 0xE6, 0xC2,
                      					0xC3, 0xC4, 0xC5, 0xC6, 0xC7 };
 
static bit LCD_RS	@ ((unsigned)&PORTE*8+0);	// Register select
static bit LCD_EN	@ ((unsigned)&PORTE*8+1);	// Enable

#define	LCD_STROBE	((LCD_EN = 1),(LCD_EN=0))


/* write a byte to the LCD in 4 bit mode */

void
lcd_write(unsigned char c)
{
	PORTB = (PORTB & 0xC0) |  (c >> 4);
	LCD_STROBE;
	PORTB = (PORTB & 0xC0) |  (c & 0x0F);
	LCD_STROBE;
	DelayUs(40);
}

/*
 * 	Clear and home the LCD
 */

void
lcd_clear(void)
{
	LCD_RS = 0;
	lcd_write(0x1);
	DelayMs(2);
}

/* write a string of chars to the LCD */

void
lcd_puts(const char * s)
{
	unsigned char c;
	LCD_RS = 1;	// write characters
	while(*s) {
		c = *s;
		if ( c > 191 ) c = russian[c-192];
		lcd_write(c);
		s++;
	}
}

/* write one character to the LCD */

void
lcd_putch(char c)
{
	if ( c > 191 ) c = russian[c-192];
	LCD_RS = 1;	// write characters
	PORTB = (PORTB & 0xC0) |  (c >> 4);
	LCD_STROBE;
	PORTB = (PORTB & 0xC0) |  (c & 0x0F);
	LCD_STROBE;
	DelayUs(40);

}


/*
 * Go to the specified position
 */

void
lcd_goto(unsigned char pos)
{
	LCD_RS = 0;
	lcd_write(0x80 | pos);
}
	
/* initialise the LCD - put into 4 bit mode */

void
lcd_init(void)
{
	LCD_RS = 0;	// write control bytes
	DelayMs(15);	// power on delay
	PORTD = 0x30;	// attention!
	LCD_STROBE;
	DelayMs(5);
	LCD_STROBE;
	DelayUs(100);
	LCD_STROBE;
	DelayMs(5);
	PORTD = 0x20;	// set 4 bit mode
	LCD_STROBE;
	DelayUs(40);
	lcd_write(0x28);	// 4 bit mode, 1/16 duty, 5x8 font
	lcd_write(0x08);	// display off
	lcd_write(0x0C);	// display on, blink curson off
	lcd_write(0x06);	// entry mode
}
